/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
/* CVS INFO
 * $Header: /var/cvs/Cricket/src/cricketdaemon/Util.java,v 1.18 2004/09/15 17:25:31 aklmiu Exp $
 */
package cricketdaemon;

import java.io.*;
import Jama.*;

/**
 * Contains a bunch of miscellaneous methods useful for debugging and
 * debug logging.
 *
 * @author Allen Miu
 */
public class Util {

    public static final boolean APPEND_OLDLOGFILE  = true;
    public static final boolean DESTROY_OLDLOGFILE = false;


    public static final long MAX_HWCLOCKVAL = (long) 2*Integer.MAX_VALUE;

    private static PrintWriter _logFile = null;

    /*
      public static final int CLOCK_FREQ_HZ  = 15625;
      public static final double TEMPERATURE_C = 20.0;
      public static final double HUMIDITY_PERCENT = 50.0;
      public static final double DELAY_OFFSET_CM = 80.0;
      public static final double SPEED_OF_SOUND = 34449.0;
    */
    
    /* Cricket receiver offset */
    /*
      public static final int BEACON_OFFSET_BODHI = 36;
      public static final int BEACON_OFFSET_CM = 80;
    */


    /*
      public static double bToCentimeters(double bodhi)
      {
      return (double) bodhi*34449.0/CLOCK_FREQ_HZ; // - DELAY_OFFSET_CM;
      }
    */

    /*
    public static double centimetersToB(double cm)
    {
	return (double) cm*CLOCK_FREQ_HZ/SPEED_OF_SOUND;
    }
    */

    public static void assert_nonjava(boolean b, String errMsg) {
	try {
	    if(!b)
		throw new Exception();
	}
	catch (Exception e) {
	    System.err.println("ASSERTION FAILED--> "+errMsg);
	    e.printStackTrace();
	    //need to close log file
	    System.exit(1);
	}
    }
    
    // logs message to some file
    public static void warn(String s)
    {
	String w = "WARNING: "+s;
	if(!CricketDaemon.debugMode)
	    System.err.println(w);
	log(w);
    }

    public static void log(String s) 
    {
	if(CricketDaemon.debugMode)
	    System.out.println(s);

	if(_logFile != null && s != null) {
	    _logFile.println(s);
	    _logFile.flush();
	}
    }

    public static void init(String file)
    {
	initLogFile(file);
    }
    
    private static void initLogFile(String fName) {
	try {
	    File f = new File(fName);
	    assert_nonjava(!f.exists(), fName+" exists, please remove or use another filename");
	    _logFile = new PrintWriter(new FileWriter(fName, DESTROY_OLDLOGFILE));
	} 
	catch (IOException e) {
	    assert_nonjava(false,"Error opening file '" + fName + "'");
	}
    }

    public static void exit() {
	log("Util::exit()");
	if(_logFile != null) {
	    _logFile.println(""); // pad a null line to separate different
	    _logFile.close();     // log sessions when the append mode is used
	}
    }

    public static int min(int a, int b)
    {
	if(b < a)
	    return b;
	return a;
    }

    public static int max(int a, int b)
    {
	if(a < b)
	    return b;
	return a;
    }


    public static String textOf(double d, int places) 
    {
	String s = String.valueOf(d);
	int endIndex = s.length();
	int decIndex = s.indexOf(".");
	if(decIndex >= 0)
	    endIndex = min(endIndex, decIndex + places + 1);
	
	return s.substring(0, endIndex);
    }

    /* java doesn't parse NaN properly, hence this function */
    public static double parseDouble(String value) 
    {
	//System.out.println("parseDouble "+value);
	if(value.equals("NaN")) 
	    return Double.NaN;
	else {
	    //	    System.out.println("parsing double");
	    double result = Double.parseDouble(value);
	    //double result = 3.0;
	    //	    System.out.println("parseDouble returning "+ result);
	    return result;
	}
    }

    // not very efficient
    public static void bufcpy(byte[] dst, byte[] src, int dstoffset, int length)
    {
	for(int i = 0; i < length; i++) {
	    dst[dstoffset+i] = src[i];
	}
    }

    public static void setDiagMatrix(Matrix m, int i, int j, int ct, double v)
    {
	for(int c = 0; c < ct; c++) {
	    m.set(i+c, j+c, v);
	}
    }
}
