/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.space;

import cricketdaemon.Util;
import cricketdaemon.reader.CartesianCoord;
import java.util.*;

/** 
 * A distance measurment from a a single beacon. Implements
 * Comparator to allow sorting by distance in an array.
 *
 * @author Allen Miu
 */
public class DistanceEstimate implements Comparator {
    public String space;
    public int beaconID;    // need to get rid of these primitives??
    public double distance;
    public CartesianCoord coord;

    public Integer beaconIDInteger;
    public Double distanceDouble;

    public DistanceEstimate(String s, int b, CartesianCoord c, double d) {
	space = s; beaconID = b; coord = c; distance = d;
	beaconIDInteger = new Integer(b);
	distanceDouble = new Double(d);
    }

    public int compare(Object o1, Object o2) {
	DistanceEstimate e1 = (DistanceEstimate) o1;
	DistanceEstimate e2 = (DistanceEstimate) o2;

	int diff = (int) (e1.distance - e2.distance);
	
	if(diff == 0) {
	    diff = e1.space.compareTo(e2.space);
	    if(diff == 0)
		return e1.beaconID - e2.beaconID;
	}
	
	return diff;
    }

    public boolean equals(Object o) {
	return compare(this, o) == 0;
    }

    public int hashCode() {
	return space.hashCode()+beaconIDInteger.hashCode()+
	    distanceDouble.hashCode();
    }
    
    public String toString() {
	return space+" "+beaconID+" "+distance;
    }
}

